@@ -6,7 +6,7 @@
 
     Standard interface implemented by Eve data layers.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 import datetime
@@ -292,16 +292,23 @@ class DataLayer(object):
 
         :param resource: resource being accessed.
 
+        .. versionchanged:: 0.5
+           If allow_unknown is enabled for the resource, don't return any
+           projection for the document. Addresses #397 and #250.
+
         .. versionchanged:: 0.4
            Return copies to avoid accidental tampering. Fix #258.
 
         .. versionchanged:: 0.2
            Support for 'default_sort'.
         """
-        source = copy(config.SOURCES[resource]['source'])
-        filter_ = copy(config.SOURCES[resource]['filter'])
-        projection = copy(config.SOURCES[resource]['projection'])
-        sort = copy(config.SOURCES[resource]['default_sort'])
+        dsource = config.SOURCES[resource]
+
+        source = copy(dsource['source'])
+        filter_ = copy(dsource['filter'])
+        sort = copy(dsource['default_sort'])
+        projection = copy(dsource['projection'])
+
         return source, filter_, projection, sort,
 
     def _datasource_ex(self, resource, query=None, client_projection=None,
@@ -309,6 +316,9 @@ class DataLayer(object):
         """ Returns both db collection and exact query (base filter included)
         to which an API resource refers to.
 
+        .. versionchanged:: 0.5
+           Let client projection work when 'allow_unknown' is active (#497).
+
         .. versionchanged:: 0.4
            Always return required/auto fields (issue 282.)
 
@@ -374,22 +384,27 @@ class DataLayer(object):
                 query = filter_
 
         fields = projection_
-        keep_fields = auto_fields(resource)
         if client_projection:
-            # only allow fields which are included with the standard projection
-            # for the resource (avoid sniffing of private fields)
-            if 0 in client_projection.values():
-                # exclusive projection - all values are 1 unless specified
+            if projection_:
+                # only allow fields which are included with the standard
+                # projection for the resource (avoid sniffing of private
+                # fields)
+                keep_fields = auto_fields(resource)
+                if 0 not in client_projection.values():
+                    # inclusive projection - all values are 0 unless spec. or
+                    # auto
+                    fields = dict([(field, field in keep_fields) for field in
+                                   fields.keys()])
                 for field, value in client_projection.items():
-                    if value == 0 and value not in keep_fields and \
-                            field in fields:
-                        del fields[field]
+                    field_base = field.split('.')[0]
+                    if field_base not in keep_fields and field_base in fields:
+                        fields[field] = value
+                fields = dict([(field, 1) for field, value in fields.items() if
+                               value])
             else:
-                # inclusive projection - all values are 0 unless spec. or auto
-                for field in list(fields.keys()):
-                    if field not in client_projection and \
-                            field not in keep_fields:
-                        del fields[field]
+                # there's no standard projection so we assume we are in a
+                # allow_unknown = True
+                fields = client_projection
 
         # If the current HTTP method is in `public_methods` or
         # `public_item_methods`, skip the `auth_field` check
