@@ -4,11 +4,10 @@
 
     The actual implementation of the MongoDB data layer.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
-import sys
 import ast
 import itertools
 from bson.errors import InvalidId
@@ -16,12 +15,13 @@ import simplejson as json
 import pymongo
 from flask import abort
 from flask.ext.pymongo import PyMongo
+from werkzeug.exceptions import HTTPException
 from bson import ObjectId
 from datetime import datetime
 from eve.io.mongo.parser import parse, ParseError
 from eve.io.base import DataLayer, ConnectionException, BaseJSONEncoder
 from eve.utils import config, debug_error_message, validate_filters, \
-    str_to_date
+    str_to_date, str_type
 
 
 class MongoJSONEncoder(BaseJSONEncoder):
@@ -42,6 +42,10 @@ class MongoJSONEncoder(BaseJSONEncoder):
 class Mongo(DataLayer):
     """ MongoDB data access layer for Eve REST API.
 
+    .. versionchanged:: 0.5
+       Properly serialize nullable float and integers. #469.
+       Return 400 if unsupported query operators are used. #387.
+
     .. versionchanged:: 0.4
        Don't serialize to objectid if value is null. #341.
 
@@ -54,13 +58,25 @@ class Mongo(DataLayer):
 
     serializers = {
         'objectid': lambda value: ObjectId(value) if value else None,
-        'datetime': str_to_date
+        'datetime': str_to_date,
+        'integer': lambda value: int(value) if value is not None else None,
+        'float': lambda value: float(value) if value is not None else None,
     }
 
-    # JSON serializer  s a class attribute. Allows extensions to replace it
+    # JSON serializer is a class attribute. Allows extensions to replace it
     # with their own implementation.
     json_encoder_class = MongoJSONEncoder
 
+    operators = set(
+        ['$gt', '$gte', '$in', '$lt', '$lt', '$lte', '$ne', '$nin'] +
+        ['$or', '$and', '$not', '$nor'] +
+        ['$mod', '$regex', '$text', '$where'] +
+        ['$options', '$search', '$language'] +
+        ['$exists', '$type'] +
+        ['$geoWithin', '$geoIntersects', '$near', '$nearSphere'] +
+        ['$all', '$elemMatch', '$size']
+    )
+
     def init_app(self, app):
         """ Initialize PyMongo.
         .. versionchanged:: 0.0.9
@@ -89,6 +105,12 @@ class Mongo(DataLayer):
         :param req: a :class:`ParsedRequest`instance.
         :param sub_resource_lookup: sub-resource lookup from the endpoint url.
 
+        .. versionchanged:: 0.5
+           Support for comma delimited sort syntax. Addresses #443.
+           Return the error if a blacklisted MongoDB operator is used in query.
+           Abort with 400 if unsupported query operator is used. #387.
+           Abort with 400 in case of invalid sort syntax. #387.
+
         .. versionchanged:: 0.4
            'allowed_filters' is now checked before adding 'sub_resource_lookup'
            to the query, as it is considered safe.
@@ -143,12 +165,31 @@ class Mongo(DataLayer):
         spec = {}
 
         if req.sort:
-            client_sort = ast.literal_eval(req.sort)
+            try:
+                # assume it's mongo syntax (ie. ?sort=[("name", 1)])
+                client_sort = ast.literal_eval(req.sort)
+            except ValueError:
+                # it's not mongo so let's see if it's a comma delimited string
+                # instead (ie. "?sort=-age, name").
+                sort = []
+                for sort_arg in [s.strip() for s in req.sort.split(",")]:
+                    if sort_arg[0] == "-":
+                        sort.append((sort_arg[1:], -1))
+                    else:
+                        sort.append((sort_arg, 1))
+                if len(sort) > 0:
+                    client_sort = sort
+            except Exception as e:
+                abort(400, description=debug_error_message(str(e)))
 
         if req.where:
             try:
                 spec = self._sanitize(json.loads(req.where))
+            except HTTPException as e:
+                # _sanitize() is raising an HTTP exception; let it fire.
+                raise
             except:
+                # couldn't parse as mongo query; give the python parser a shot.
                 try:
                     spec = parse(req.where)
                 except ParseError:
@@ -312,6 +353,14 @@ class Mongo(DataLayer):
         try:
             return self.driver.db[datasource].insert(doc_or_docs,
                                                      **self._wc(resource))
+        except pymongo.errors.DuplicateKeyError as e:
+            abort(409, description=debug_error_message(
+                'pymongo.errors.DuplicateKeyError: %s' % e
+            ))
+        except pymongo.errors.InvalidOperation as e:
+            abort(500, description=debug_error_message(
+                'pymongo.errors.InvalidOperation: %s' % e
+            ))
         except pymongo.errors.OperationFailure as e:
             # most likely a 'w' (write_concern) setting which needs an
             # existing ReplicaSet which doesn't exist. Please note that the
@@ -558,11 +607,6 @@ class Mongo(DataLayer):
 
         .. versionadded:: 0.0.4
         """
-        if sys.version_info[0] == 3:
-            _str_type = str
-        else:
-            _str_type = basestring  # noqa
-
         schema = config.DOMAIN[resource]
         skip_objectid = schema.get('query_objectid_as_string', False)
 
@@ -572,7 +616,16 @@ class Mongo(DataLayer):
             except:
                 if not skip_objectid:
                     try:
-                        return ObjectId(v)
+                        # Convert to unicode because ObjectId() interprets
+                        # 12-character strings (but not unicode) as binary
+                        # representations of ObjectId's.  See
+                        # https://github.com/nicolaiarocci/eve/issues/508
+                        try:
+                            r = ObjectId(unicode(v))
+                        except NameError:
+                            # We're on Python 3 so it's all unicode # already.
+                            r = ObjectId(v)
+                        return r
                     except:
                         return v
                 else:
@@ -587,7 +640,7 @@ class Mongo(DataLayer):
                         source[k][i] = self._mongotize(v1, resource)
                     else:
                         source[k][i] = try_cast(v1)
-            elif isinstance(v, _str_type):
+            elif isinstance(v, str_type):
                 source[k] = try_cast(v)
 
         return source
@@ -596,23 +649,34 @@ class Mongo(DataLayer):
         """ Makes sure that only allowed operators are included in the query,
         aborts with a 400 otherwise.
 
+        .. versionchanged:: 0.5
+           Abort with 400 if unsupported query operators are used. #387.
+           DRY.
+
         .. versionchanged:: 0.0.9
            More informative error messages.
            Allow ``auth_username_field`` to be set to ``ID_FIELD``.
 
         .. versionadded:: 0.0.7
         """
-        if set(spec.keys()) & set(config.MONGO_QUERY_BLACKLIST):
-            abort(400, description=debug_error_message(
-                'Query contains operators banned in MONGO_QUERY_BLACKLIST'
-            ))
+        def sanitize_keys(spec):
+            ops = set([op for op in spec.keys() if op[0] == '$'])
+            unknown = ops - Mongo.operators
+            if unknown:
+                abort(400, description=debug_error_message(
+                    'Query contains unknown or unsupported operators: %s' %
+                    ', '.join(unknown)
+                ))
+
+            if set(spec.keys()) & set(config.MONGO_QUERY_BLACKLIST):
+                abort(400, description=debug_error_message(
+                    'Query contains operators banned in MONGO_QUERY_BLACKLIST'
+                ))
+
+        sanitize_keys(spec)
         for value in spec.values():
             if isinstance(value, dict):
-                if set(value.keys()) & set(config.MONGO_QUERY_BLACKLIST):
-                    abort(400, description=debug_error_message(
-                        'Query contains operators banned '
-                        'in MONGO_QUERY_BLACKLIST'
-                    ))
+                sanitize_keys(value)
         return spec
 
     def _wc(self, resource):
@@ -633,6 +697,9 @@ class Mongo(DataLayer):
         if req and req.projection:
             try:
                 client_projection = json.loads(req.projection)
+                if not isinstance(client_projection, dict):
+                    raise Exception('The projection parameter has to be a '
+                                    'dict')
             except:
                 abort(400, description=debug_error_message(
                     'Unable to parse `projection` clause'
