@@ -6,7 +6,7 @@
 
     Utility functions for API methods implementations.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -22,15 +22,21 @@ from eve.versioning import resolve_document_version, \
     get_data_version_relation_document, missing_version_field
 
 
-def get_document(resource, **lookup):
+def get_document(resource, concurrency_check, **lookup):
     """ Retrieves and return a single document. Since this function is used by
     the editing methods (POST, PATCH, DELETE), we make sure that the client
     request references the current representation of the document before
-    returning it.
+    returning it. However, this concurrency control may be turned off by
+    internal functions.
 
     :param resource: the name of the resource to which the document belongs to.
+    :param concurrency_check: boolean check for concurrency control
     :param **lookup: document lookup query
 
+    .. versionchanged:: 0.5
+       Concurrency control optional for internal functions.
+       ETAG are now stored with the document (#369).
+
     .. versionchanged:: 0.0.9
        More informative error messages.
 
@@ -42,7 +48,7 @@ def get_document(resource, **lookup):
     document = app.data.find_one(resource, None, **lookup)
     if document:
 
-        if not req.if_match and config.IF_MATCH:
+        if not req.if_match and config.IF_MATCH and concurrency_check:
             # we don't allow editing unless the client provides an etag
             # for the document
             abort(403, description=debug_error_message(
@@ -54,12 +60,14 @@ def get_document(resource, **lookup):
         document[config.LAST_UPDATED] = last_updated(document)
         document[config.DATE_CREATED] = date_created(document)
 
-        if req.if_match and req.if_match != document_etag(document):
-            # client and server etags must match, or we don't allow editing
-            # (ensures that client's version of the document is up to date)
-            abort(412, description=debug_error_message(
-                'Client and server etags don\'t match'
-            ))
+        if req.if_match and concurrency_check:
+            etag = document.get(config.ETAG, document_etag(document))
+            if req.if_match != etag:
+                # client and server etags must match, or we don't allow editing
+                # (ensures that client's version of the document is up to date)
+                abort(412, description=debug_error_message(
+                    'Client and server etags don\'t match'
+                ))
 
     return document
 
@@ -301,10 +309,10 @@ def serialize(document, resource=None, schema=None, fields=None):
         for field in fields:
             if field in schema:
                 field_schema = schema[field]
-                field_type = field_schema['type']
+                field_type = field_schema.get('type')
                 if 'schema' in field_schema:
                     field_schema = field_schema['schema']
-                    if 'dict' in (field_type, field_schema.get('type', '')):
+                    if 'dict' in (field_type, field_schema.get('type')):
                         # either a dict or a list of dicts
                         embedded = [document[field]] if field_type == 'dict' \
                             else document[field]
@@ -315,8 +323,8 @@ def serialize(document, resource=None, schema=None, fields=None):
                             else:
                                 serialize(subdocument, schema=field_schema)
                     else:
-                        # a list of one type, arbirtrary length
-                        field_type = field_schema['type']
+                        # a list of one type, arbitrary length
+                        field_type = field_schema.get('type')
                         if field_type in app.data.serializers:
                             for i, v in enumerate(document[field]):
                                 document[field][i] = \
@@ -325,7 +333,7 @@ def serialize(document, resource=None, schema=None, fields=None):
                     # a list of multiple types, fixed length
                     for i, (s, v) in enumerate(zip(field_schema['items'],
                                                    document[field])):
-                        field_type = s['type'] if 'type' in s else None
+                        field_type = s.get('type')
                         if field_type in app.data.serializers:
                             document[field][i] = \
                                 app.data.serializers[field_type](
@@ -347,6 +355,10 @@ def build_response_document(
     :param embedded_fields: the list of fields we are allowed to embed.
     :param document: the latest version of document.
 
+    .. versionchanged:: 0.5
+       Only compute ETAG if necessary (#369).
+       Add version support (#475).
+
     .. versionadded:: 0.4
     """
     # need to update the document field since the etag must be computed on the
@@ -356,15 +368,20 @@ def build_response_document(
     document[config.LAST_UPDATED] = last_updated(document)
     # TODO: last_update could include consideration for embedded documents
 
-    # generate ETag
-    if config.IF_MATCH:
+    # Up to v0.4 etags were not stored with the documents.
+    if config.IF_MATCH and config.ETAG not in document:
         document[config.ETAG] = document_etag(document)
 
     # hateoas links
     if config.DOMAIN[resource]['hateoas'] and config.ID_FIELD in document:
+        version = None
+        if config.DOMAIN[resource]['versioning'] is True \
+                and request.args.get(config.VERSION_PARAM):
+            version = document[config.VERSION]
         document[config.LINKS] = {'self':
                                   document_link(resource,
-                                                document[config.ID_FIELD])}
+                                                document[config.ID_FIELD],
+                                                version)}
 
     # add version numbers
     resolve_document_version(document, resource, 'GET', latest_doc)
@@ -374,6 +391,30 @@ def build_response_document(
     resolve_embedded_documents(document, resource, embedded_fields)
 
 
+def field_definition(resource, chained_fields):
+    """ Resolves query string to resource with dot notation like
+    'people.address.city' and returns corresponding field definition
+    of the resource
+
+    :param resource: the resource name whose field to be accepted.
+    :param chained_fields: query string to retrieve field definition
+
+    .. versionadded 0.5
+    """
+    definition = config.DOMAIN[resource]
+    subfields = chained_fields.split('.')
+
+    for field in subfields:
+        try:
+            definition = definition['schema'][field]
+            if definition['type'] == 'list':
+                definition = definition['schema']
+        except KeyError:
+            return None
+
+    return definition
+
+
 def resolve_embedded_fields(resource, req):
     """ Returns a list of validated embedded fields from the incoming request
     or from the resource definition is the request does not specify.
@@ -381,6 +422,9 @@ def resolve_embedded_fields(resource, req):
     :param resource: the resource name.
     :param req: and instace of :class:`eve.utils.ParsedRequest`.
 
+    .. versionchanged:: 0.5
+       Enables subdocuments embedding. #389.
+
     .. versionadded:: 0.4
     """
     embedded_fields = []
@@ -413,29 +457,115 @@ def resolve_embedded_fields(resource, req):
     enabled_embedded_fields = []
     for field in embedded_fields:
         # Reject bogus field names
-        if field in config.DOMAIN[resource]['schema']:
-            field_definition = config.DOMAIN[resource]['schema'][field]
-            if 'data_relation' in field_definition and \
-                    field_definition['data_relation'].get('embeddable'):
+        field_def = field_definition(resource, field)
+        if field_def:
+            if field_def['type'] == 'list':
+                field_def = field_def['schema']
+            if 'data_relation' in field_def and \
+                    field_def['data_relation'].get('embeddable'):
                 # or could raise 400 here
                 enabled_embedded_fields.append(field)
 
     return enabled_embedded_fields
 
 
+def embedded_document(reference, data_relation, field_name):
+    """ Returns a document to be embedded by reference using data_relation
+    taking into account document versions
+
+    :param reference: reference to the document to be embedded.
+    :param data_relation: the relation schema definition.
+    :param field_name: field name used in abort message only
+
+    .. versionadded:: 0.5
+    """
+    # Retrieve and serialize the requested document
+    if 'version' in data_relation and data_relation['version'] is True:
+        # support late versioning
+        if reference[config.VERSION] == 1:
+            # there is a chance this document hasn't been saved
+            # since versioning was turned on
+            embedded_doc = missing_version_field(data_relation, reference)
+
+            if embedded_doc is None:
+                # this document has been saved since the data_relation was
+                # made - we basically do not have the copy of the document
+                # that existed when the data relation was made, but we'll
+                # try the next best thing - the first version
+                reference[config.VERSION] = 1
+                embedded_doc = get_data_version_relation_document(
+                    data_relation, reference)
+
+            latest_embedded_doc = embedded_doc
+        else:
+            # grab the specific version
+            embedded_doc = get_data_version_relation_document(
+                data_relation, reference)
+
+            # grab the latest version
+            latest_embedded_doc = get_data_version_relation_document(
+                data_relation, reference, latest=True)
+
+        # make sure we got the documents
+        if embedded_doc is None or latest_embedded_doc is None:
+            # your database is not consistent!!! that is bad
+            abort(404, description=debug_error_message(
+                "Unable to locate embedded documents for '%s'" %
+                field_name
+            ))
+
+        # build the response document
+        build_response_document(embedded_doc, data_relation['resource'],
+                                [], latest_embedded_doc)
+    else:
+        subresource = data_relation['resource']
+        embedded_doc = app.data.find_one(subresource, None,
+                                         **{config.ID_FIELD: reference})
+        if embedded_doc:
+            resolve_media_files(embedded_doc, subresource)
+
+    return embedded_doc
+
+
+def subdocuments(fields_chain, document):
+    """ Traverses the given document and yields subdocuments which
+    correspond to the given fields_chain
+
+    :param fields_chain: list of nested field names.
+    :param document: document to be traversed
+
+    .. versionadded:: 0.5
+    """
+    if len(fields_chain) == 0:
+        yield document
+    else:
+        subdocument = document[fields_chain[0]]
+        docs = subdocument if isinstance(subdocument, list) else [subdocument]
+        for doc in docs:
+            for result in subdocuments(fields_chain[1:], doc):
+                yield result
+
+
 def resolve_embedded_documents(document, resource, embedded_fields):
     """ Loops through the documents, adding embedded representations
     of any fields that are (1) defined eligible for embedding in the
     DOMAIN and (2) requested to be embedded in the current `req`.
 
-    Currently we only support a single layer of embedding,
-    i.e. /invoices/?embedded={"user":1}
-    *NOT*  /invoices/?embedded={"user.friends":1}
+    Currently we support embedding of documents by references located
+    in any subdocuments. For example, query embedded={"user.friends":1}
+    will return a document with "user" and all his "friends" embedded,
+    but only if "user" is a subdocument and "friends" is a list of
+    references.
+    We do not support multiple layers embeddings.
 
     :param document: the document to embed other documents into.
     :param resource: the resource name.
     :param embedded_fields: the list of fields we are allowed to embed.
 
+    .. versionchagend:: 0.5
+       Support for embedding documents located in subdocuments.
+       Allocated two functions embedded_document and subdocuments.
+
     .. versionchagend:: 0.4
         Moved parsing of embedded fields to _resolve_embedded_fields.
         Support for document versioning.
@@ -448,56 +578,19 @@ def resolve_embedded_documents(document, resource, embedded_fields):
 
     .. versionadded:: 0.1.0
     """
-    schema = config.DOMAIN[resource]['schema']
     for field in embedded_fields:
-        data_relation = schema[field]['data_relation']
-        # Retrieve and serialize the requested document
-        if 'version' in data_relation and data_relation['version'] is True:
-            # support late versioning
-            if document[field][config.VERSION] == 1:
-                # there is a chance this document hasn't been saved
-                # since versioning was turned on
-                embedded_doc = missing_version_field(
-                    data_relation, document[field])
-
-                if embedded_doc is None:
-                    # this document has been saved since the data_relation was
-                    # made - we basically do not have the copy of the document
-                    # that existed when the data relation was made, but we'll
-                    # try the next best thing - the first version
-                    document[field][config.VERSION] = 1
-                    embedded_doc = get_data_version_relation_document(
-                        data_relation, document[field])
-
-                latest_embedded_doc = embedded_doc
+        data_relation = field_definition(resource, field)['data_relation']
+        getter = lambda ref: embedded_document(ref, data_relation, field)  # noqa
+        fields_chain = field.split('.')
+        last_field = fields_chain[-1]
+        for subdocument in subdocuments(fields_chain[:-1], document):
+            if last_field not in subdocument:
+                continue
+            if isinstance(subdocument[last_field], list):
+                subdocument[last_field] = list(map(getter,
+                                                   subdocument[last_field]))
             else:
-                # grab the specific version
-                embedded_doc = get_data_version_relation_document(
-                    data_relation, document[field])
-
-                # grab the latest version
-                latest_embedded_doc = get_data_version_relation_document(
-                    data_relation, document[field], latest=True)
-
-            # make sure we got the documents
-            if embedded_doc is None or latest_embedded_doc is None:
-                # your database is not consistent!!! that is bad
-                abort(404, description=debug_error_message(
-                    "Unable to locate embedded documents for '%s'" %
-                    field
-                ))
-
-            # build the response document
-            build_response_document(
-                embedded_doc, data_relation['resource'],
-                [], latest_embedded_doc)
-        else:
-            embedded_doc = app.data.find_one(
-                data_relation['resource'], None,
-                **{config.ID_FIELD: document[field]}
-            )
-        if embedded_doc:
-            document[field] = embedded_doc
+                subdocument[last_field] = getter(subdocument[last_field])
 
 
 def resolve_media_files(document, resource):
@@ -548,15 +641,27 @@ def marshal_write_response(document, resource):
     :param document: the response document.
     :param resource: the resource being consumed by the request.
 
+    .. versionchanged: 0.5
+       Avoid exposing 'auth_field' if it is not intended to be public.
+
     .. versionadded:: 0.4
     """
 
+    resource_def = app.config['DOMAIN'][resource]
     if app.config['BANDWIDTH_SAVER'] is True:
         # only return the automatic fields and special extra fields
-        fields = auto_fields(resource) + \
-            app.config['DOMAIN'][resource]['extra_response_fields']
+        fields = auto_fields(resource) + resource_def['extra_response_fields']
         document = dict((k, v) for (k, v) in document.items() if k in fields)
-
+    else:
+        # avoid exposing the auth_field if it is not included in the
+        # resource schema.
+        auth_field = resource_def.get('auth_field')
+        if auth_field and auth_field not in resource_def['schema']:
+            try:
+                del(document[auth_field])
+            except:
+                # 'auth_field' value has not been set by the auth class.
+                pass
     return document
 
 
@@ -578,15 +683,16 @@ def store_media_files(document, resource, original=None):
     # document update fails we should probably attempt a cleanup on the storage
     # sytem. Easier said than done though.
     for field in resource_media_fields(document, resource):
-        if original and hasattr(original, field):
+        if original and field in original:
             # since file replacement is not supported by the media storage
             # system, we first need to delete the file being replaced.
             app.media.delete(original[field])
 
-        # store file and update document with file's unique id/filename
-        # also pass in mimetype for use when retrieving the file
-        document[field] = app.media.put(document[field],
-                                        content_type=document[field].mimetype)
+        if document[field]:
+            # store file and update document with file's unique id/filename
+            # also pass in mimetype for use when retrieving the file
+            document[field] = app.media.put(
+                document[field], content_type=document[field].mimetype)
 
 
 def resource_media_fields(document, resource):
@@ -638,6 +744,19 @@ def resolve_user_restricted_access(document, resource):
             document[auth_field] = request_auth_value
 
 
+def resolve_document_etag(documents):
+    """ Adds etags to documents.
+
+    .. versionadded:: 0.5
+    """
+    if config.IF_MATCH:
+        if not isinstance(documents, list):
+            documents = [documents]
+
+        for document in documents:
+            document[config.ETAG] = document_etag(document)
+
+
 def pre_event(f):
     """ Enable a Hook pre http request.
 
@@ -677,11 +796,15 @@ def pre_event(f):
     return decorated
 
 
-def document_link(resource, document_id):
+def document_link(resource, document_id, version=None):
     """ Returns a link to a document endpoint.
 
     :param resource: the resource name.
     :param document_id: the document unique identifier.
+    :param version: the document version. Defaults to None.
+
+    .. versionchanged:: 0.5
+       Add version support (#475).
 
     .. versionchanged:: 0.4
        Use the regex-neutral resource_link function.
@@ -692,23 +815,102 @@ def document_link(resource, document_id):
     .. versionchanged:: 0.0.3
        Now returning a JSON link
     """
+    version_part = '?version=%s' % version if version else ''
     return {'title': '%s' % config.DOMAIN[resource]['item_title'],
-            'href': '%s/%s' % (resource_link(), document_id)}
+            'href': '%s/%s%s' % (resource_link(), document_id, version_part)}
 
 
 def resource_link():
-    """ Returns the current resource path complete with server name if
-    available. Mostly going to be used by hatoeas functions when building
+    """ Returns the current resource path relative to the API entry point.
+    Mostly going to be used by hatoeas functions when building
     document/resource links. The resource URL stored in the config settings
     might contain regexes and custom variable names, all of which are not
     needed in the response payload.
 
+    .. versionchanged:: 0.5
+       URL is relative to API root.
+
     .. versionadded:: 0.4
     """
-    path = request.path.rstrip('/')
+    path = request.path.strip('/')
+
     if '|item' in request.endpoint:
         path = path[:path.rfind('/')]
-    server_name = config.SERVER_NAME if config.SERVER_NAME else ''
-    if config.URL_PROTOCOL:
-        server_name = '%s://%s' % (config.URL_PROTOCOL, server_name)
-    return '%s%s' % (server_name, path)
+
+    def strip_prefix(hit):
+        return path[len(hit):] if path.startswith(hit) else path
+
+    if config.URL_PREFIX:
+        path = strip_prefix(config.URL_PREFIX + '/')
+    if config.API_VERSION:
+        path = strip_prefix(config.API_VERSION + '/')
+    return path
+
+
+def oplog_push(resource, updates, op, id=None):
+    """ Pushes an edit operation to the oplog if included in OPLOG_METHODS. To
+    save on storage space (at least on MongoDB) field names are shortened:
+
+        'r' = resource endpoint,
+        'o' = operation performed,
+        'i' = unique id of the document involved,
+        'pi' = client IP,
+        'c' = changes
+
+    config.LAST_UPDATED, config.LAST_CREATED and AUTH_FIELD are not being
+    shortened to allow for standard endpoint behavior (so clients can
+    query the endpoint with If-Modified-Since queries, and User-Restricted-
+    Resource-Access will keep working on the oplog endpoint too).
+
+    :param resource: name of the resource involved.
+    :param updates: updates performed with the edit operation.
+    :param op: operation performed. Can be 'POST', 'PUT', 'PATCH', 'DELETE'.
+    :param id: unique id of the document.
+
+    .. versionadded:: 0.5
+    """
+    if not config.OPLOG or op not in config.OPLOG_METHODS:
+        return
+
+    if updates is None:
+        updates = {}
+
+    if not isinstance(updates, list):
+        updates = [updates]
+
+    entries = []
+    for update in updates:
+        entry = {
+            'r': config.URLS[resource],
+            'o': op,
+            'i': update[config.ID_FIELD] if config.ID_FIELD in update else id,
+        }
+        if config.LAST_UPDATED in update:
+            last_update = update[config.LAST_UPDATED]
+        else:
+            last_update = datetime.utcnow().replace(microsecond=0)
+        entry[config.LAST_UPDATED] = entry[config.DATE_CREATED] = last_update
+        if config.OPLOG_AUDIT:
+
+            # TODO this needs further investigation. See:
+            # http://esd.io/blog/flask-apps-heroku-real-ip-spoofing.html;
+            # https://stackoverflow.com/questions/22868900/how-do-i-safely-get-the-users-real-ip-address-in-flask-using-mod-wsgi
+            entry['ip'] = request.remote_addr
+
+            if op in ('PATCH', 'PUT', 'DELETE'):
+                # these fields are already contained in 'entry'.
+                del(update[config.LAST_UPDATED])
+                # legacy documents (v0.4 or less) could be missing the etag
+                # field
+                if config.ETAG in update:
+                    del(update[config.ETAG])
+                entry['c'] = update
+            else:
+                pass
+
+        resolve_user_restricted_access(entry, config.OPLOG_NAME)
+
+        entries.append(entry)
+
+    if entries:
+        app.data.insert(config.OPLOG_NAME, entries)
