@@ -6,14 +6,14 @@
 
     This module imlements the DELETE method.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
 from flask import current_app as app, abort
 from eve.utils import config
 from eve.auth import requires_auth
-from eve.methods.common import get_document, ratelimit, pre_event
+from eve.methods.common import get_document, ratelimit, pre_event, oplog_push
 from eve.versioning import versioned_id_field
 
 
@@ -21,12 +21,33 @@ from eve.versioning import versioned_id_field
 @requires_auth('item')
 @pre_event
 def deleteitem(resource, **lookup):
-    """ Deletes a resource item. Deletion will occur only if request ETag
-    matches the current representation of the item.
+    """
+    Default function for handling DELETE requests, it has decorators for
+    rate limiting, authentication and for raising pre-request events.
+    After the decorators are applied forwards to call to
+    :func:`deleteitem_internal`
+
+    .. versionchanged:: 0.5
+       Split into deleteitem() and deleteitem_internal().
+    """
+    return deleteitem_internal(resource, concurrency_check=True, **lookup)
+
+
+def deleteitem_internal(resource, concurrency_check=False, **lookup):
+    """ Intended for internal delete calls, this method is not rate limited,
+    authentication is not checked, pre-request events are not raised, and
+    concurrency checking is optional. Deletes a resource item.
 
     :param resource: name of the resource to which the item(s) belong.
+    :param concurrency_check: concurrency check switch (bool)
     :param **lookup: item lookup query.
 
+    .. versionchanged:: 0.5
+       Return 204 NoContent instead of 200.
+       Push updates to OpLog.
+       Original deleteitem() has been split into deleteitem() and
+       deleteitem_internal().
+
     .. versionchanged:: 0.4
        Fix #284: If you have a media field, and set datasource projection to
        0 for that field, the media will not be deleted.
@@ -52,7 +73,7 @@ def deleteitem(resource, **lookup):
     .. versionchanged:: 0.0.4
        Added the ``requires_auth`` decorator.
     """
-    original = get_document(resource, **lookup)
+    original = get_document(resource, concurrency_check, **lookup)
     if not original:
         abort(404)
 
@@ -78,7 +99,12 @@ def deleteitem(resource, **lookup):
         if field in original:
             app.media.delete(original[field])
 
-    app.data.remove(resource, {config.ID_FIELD: original[config.ID_FIELD]})
+    id = original[config.ID_FIELD]
+    app.data.remove(resource, {config.ID_FIELD: id})
+
+    # update oplog if needed
+    oplog_push(resource, original, 'DELETE', id)
+
     # TODO: should attempt to delete version collection even if setting is off
     if app.config['DOMAIN'][resource]['versioning'] is True:
         app.data.remove(
@@ -88,7 +114,7 @@ def deleteitem(resource, **lookup):
     getattr(app, "on_deleted_item")(resource, original)
     getattr(app, "on_deleted_item_%s" % resource)(original)
 
-    return {}, None, None, 200
+    return {}, None, None, 204
 
 
 @requires_auth('resource')
@@ -97,6 +123,9 @@ def delete(resource, **lookup):
     """ Deletes all item of a resource (collection in MongoDB terms). Won't
     drop indexes. Use with caution!
 
+    .. versionchanged:: 0.5
+       Return 204 NoContent instead of 200.
+
     .. versionchanged:: 0.4
        Support for document versioning.
        'on_delete_resource' raised before performing the actual delete.
@@ -126,4 +155,4 @@ def delete(resource, **lookup):
     getattr(app, "on_deleted_resource")(resource)
     getattr(app, "on_deleted_resource_%s" % resource)()
 
-    return {}, None, None, 200
+    return {}, None, None, 204
