@@ -6,7 +6,7 @@
 
     This module imlements the PUT method.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -19,7 +19,8 @@ from flask import current_app as app, abort
 from eve.utils import config, debug_error_message, parse_request
 from eve.methods.common import get_document, parse, payload as payload_, \
     ratelimit, pre_event, store_media_files, resolve_user_restricted_access, \
-    resolve_embedded_fields, build_response_document, marshal_write_response
+    resolve_embedded_fields, build_response_document, marshal_write_response, \
+    resolve_document_etag, oplog_push
 from eve.versioning import resolve_document_version, \
     insert_versioning_documents, late_versioning_catch
 
@@ -27,15 +28,52 @@ from eve.versioning import resolve_document_version, \
 @ratelimit()
 @requires_auth('item')
 @pre_event
-def put(resource, **lookup):
-    """ Perform a document replacement. Updates are first validated against
-    the resource schema. If validation passes, the document is repalced and
-    an OK status update is returned. If validation fails a set of validation
-    issues is returned.
+def put(resource, payload=None, **lookup):
+    """
+    Default function for handling PUT requests, it has decorators for
+    rate limiting, authentication and for raising pre-request events.
+    After the decorators are applied forwards to call to :func:`put_internal`
+
+    .. versionchanged:: 0.5
+       Split into put() and put_internal().
+    """
+    return put_internal(resource, payload, concurrency_check=True,
+                        skip_validation=False, **lookup)
+
+
+def put_internal(resource, payload=None, concurrency_check=False,
+                 skip_validation=False, **lookup):
+    """ Intended for internal put calls, this method is not rate limited,
+    authentication is not checked, pre-request events are not raised, and
+    concurrency checking is optional. Performs a document replacement.
+    Updates are first validated against the resource schema. If validation
+    passes, the document is repalced and an OK status update is returned.
+    If validation fails a set of validation issues is returned.
 
     :param resource: the name of the resource to which the document belongs.
+    :param payload: alternative payload. When calling put() from your own code
+                    you can provide an alternative payload. This can be useful,
+                    for example, when you have a callback function hooked to a
+                    certain endpoint, and want to perform additional put()
+                    callsfrom there.
+
+                    Please be advised that in order to successfully use this
+                    option, a request context must be available.
+    :param concurrency_check: concurrency check switch (bool)
+    :param skip_validation: skip payload validation before write (bool)
     :param **lookup: document lookup query.
 
+    .. versionchanged:: 0.5
+       Back to resolving default values after validaton as now the validator
+       can properly validate dependency even when some have default values. See
+       #353.
+       Original put() has been split into put() and put_internal().
+       You can now pass a pre-defined custom payload to the funcion.
+       ETAG is now stored with the document (#369).
+       Catching all HTTPExceptions and returning them to the caller, allowing
+       for eventual flask.abort() invocations in callback functions to go
+       through. Fixes #395.
+
     .. versionchanged:: 0.4
        Allow abort() to be inoked by callback functions.
        Resolve default values before validation is performed. See #353.
@@ -65,10 +103,13 @@ def put(resource, **lookup):
     """
     resource_def = app.config['DOMAIN'][resource]
     schema = resource_def['schema']
-    validator = app.validator(schema, resource)
+    if not skip_validation:
+        validator = app.validator(schema, resource)
 
-    payload = payload_()
-    original = get_document(resource, **lookup)
+    if payload is None:
+        payload = payload_()
+
+    original = get_document(resource, concurrency_check, **lookup)
     if not original:
         # not found
         abort(404)
@@ -88,8 +129,10 @@ def put(resource, **lookup):
 
     try:
         document = parse(payload, resource)
-        resolve_default_values(document, resource_def['defaults'])
-        validation = validator.validate_replace(document, object_id)
+        if skip_validation:
+            validation = True
+        else:
+            validation = validator.validate_replace(document, object_id)
         if validation:
             # sneak in a shadow copy if it wasn't already there
             late_versioning_catch(original, resource)
@@ -106,6 +149,7 @@ def put(resource, **lookup):
                 document[config.ID_FIELD] = object_id
 
             resolve_user_restricted_access(document, resource)
+            resolve_default_values(document, resource_def['defaults'])
             store_media_files(document, resource, original)
             resolve_document_version(document, resource, 'PUT', original)
 
@@ -113,8 +157,14 @@ def put(resource, **lookup):
             getattr(app, "on_replace")(resource, document, original)
             getattr(app, "on_replace_%s" % resource)(document, original)
 
+            resolve_document_etag(document)
+
             # write to db
             app.data.replace(resource, object_id, document)
+
+            # update oplog if needed
+            oplog_push(resource, document, 'PUT')
+
             insert_versioning_documents(resource, document)
 
             # notify callbacks
@@ -131,8 +181,7 @@ def put(resource, **lookup):
         # TODO should probably log the error and abort 400 instead (when we
         # got logging)
         issues['validator exception'] = str(e)
-    except (exceptions.InternalServerError, exceptions.Unauthorized,
-            exceptions.Forbidden, exceptions.NotFound) as e:
+    except exceptions.HTTPException as e:
         raise e
     except Exception as e:
         # consider all other exceptions as Bad Requests
@@ -143,10 +192,12 @@ def put(resource, **lookup):
     if len(issues):
         response[config.ISSUES] = issues
         response[config.STATUS] = config.STATUS_ERR
+        status = config.VALIDATION_ERROR_STATUS
     else:
         response[config.STATUS] = config.STATUS_OK
+        status = 200
 
     # limit what actually gets sent to minimize bandwidth usage
     response = marshal_write_response(response, resource)
 
-    return response, last_modified, etag, 200
+    return response, last_modified, etag, status
