@@ -157,8 +157,11 @@ class TestGridFSMediaStorage(TestBase):
         _id = r[ID_FIELD]
         etag = r[ETAG]
 
-        # retrieve media_id and compare original and returned data
-        media_id = self.assertMediaField(_id, self.encoded, self.clean)
+        # compare original and returned data
+        self.assertMediaField(_id, self.encoded, self.clean)
+
+        # retrieve media_id
+        media_id = self.assertMediaStored(_id)
 
         # PUT replaces the file with new one
         clean = b'my new file contents'
@@ -189,8 +192,11 @@ class TestGridFSMediaStorage(TestBase):
         _id = r[ID_FIELD]
         etag = r[ETAG]
 
-        # retrieve media_id and compare original and returned data
-        media_id = self.assertMediaField(_id, self.encoded, self.clean)
+        # compare original and returned data
+        self.assertMediaField(_id, self.encoded, self.clean)
+
+        # retrieve media_id
+        media_id = self.assertMediaStored(_id)
 
         # PATCH replaces the file with new one
         clean = b'my new file contents'
@@ -213,13 +219,36 @@ class TestGridFSMediaStorage(TestBase):
         # previous media doesn't exist anymore (it's been deleted)
         self.assertFalse(self.app.media.exists(media_id))
 
+    def test_gridfs_media_storage_patch_null(self):
+        # set 'media' field to 'nullable'
+        self.domain[self.known_resource]['schema']['media']['nullable'] = True
+
+        response, status = self._post()
+        self.assert201(status)
+
+        _id = response[ID_FIELD]
+        etag = response[ETAG]
+
+        # test that nullable media field can be set to None
+        data = {'media': None}
+        headers = [('If-Match', etag)]
+        response, status = self.patch(('%s/%s' % (self.url, _id)), data=data,
+                                      headers=headers)
+        self.assert200(status)
+
+        response, status = self.get(self.known_resource, item=_id)
+        self.assert200(status)
+        self.assertEqual(response['media'], None)
+
     def test_gridfs_media_storage_delete(self):
         r, s = self._post()
         _id = r[ID_FIELD]
         etag = r[ETAG]
 
         # retrieve media_id and compare original and returned data
-        media_id = self.assertMediaField(_id, self.encoded, self.clean)
+        self.assertMediaField(_id, self.encoded, self.clean)
+
+        media_id = self.assertMediaStored(_id)
 
         # DELETE deletes both the document and the media file
         headers = [('If-Match', etag)]
@@ -227,7 +256,7 @@ class TestGridFSMediaStorage(TestBase):
         r, s = self.parse_response(
             self.test_client.delete(('%s/%s' % (self.url, _id)),
                                     headers=headers))
-        self.assert200(s)
+        self.assert204(s)
 
         # media doesn't exist anymore (it's been deleted)
         self.assertFalse(self.app.media.exists(media_id))
@@ -261,7 +290,7 @@ class TestGridFSMediaStorage(TestBase):
         r, s = self.parse_response(
             self.test_client.delete(('%s/%s' % (self.url, _id)),
                                     headers=headers))
-        self.assert200(s)
+        self.assert204(s)
 
         # media doesn't exist anymore (it's been deleted)
         self.assertFalse(self.app.media.exists(media_id))
