@@ -1,10 +1,13 @@
+import base64
+from io import BytesIO
 import simplejson as json
 from datetime import datetime
 from bson import ObjectId
+from werkzeug.datastructures import ImmutableMultiDict
 from eve.tests import TestBase
 from eve.tests.utils import DummyEvent
 from eve.tests.test_settings import MONGO_DBNAME
-from eve.utils import date_to_str, str_to_date
+from eve.utils import str_to_date, date_to_rfc1123
 
 
 class TestGet(TestBase):
@@ -37,6 +40,14 @@ class TestGet(TestBase):
         resource = response['_items']
         self.assertEqual(len(resource), self.app.config['PAGINATION_LIMIT'])
 
+    def test_get_custom_max_results(self):
+        self.app.config['QUERY_MAX_RESULTS'] = 'size'
+        maxr = 10
+        response, status = self.get(self.known_resource, '?size=%d' % maxr)
+        self.assert200(status)
+        resource = response['_items']
+        self.assertEqual(len(resource), maxr)
+
     def test_get_page(self):
         response, status = self.get(self.known_resource)
         self.assert200(status)
@@ -47,8 +58,7 @@ class TestGet(TestBase):
         self.assertPagination(response, 1, 101, 25)
 
         page = 1
-        response, status = self.get(self.known_resource,
-                                    '?page=%d' % page)
+        response, status = self.get(self.known_resource, '?page=%d' % page)
         self.assert200(status)
 
         links = response['_links']
@@ -57,8 +67,7 @@ class TestGet(TestBase):
         self.assertPagination(response, 1, 101, 25)
 
         page = 2
-        response, status = self.get(self.known_resource,
-                                    '?page=%d' % page)
+        response, status = self.get(self.known_resource, '?page=%d' % page)
         self.assert200(status)
 
         links = response['_links']
@@ -68,8 +77,7 @@ class TestGet(TestBase):
         self.assertPagination(response, 2, 101, 25)
 
         page = 5
-        response, status = self.get(self.known_resource,
-                                    '?page=%d' % page)
+        response, status = self.get(self.known_resource, '?page=%d' % page)
         self.assert200(status)
 
         links = response['_links']
@@ -77,17 +85,27 @@ class TestGet(TestBase):
         self.assertLastLink(links, None)
         self.assertPagination(response, 5, 101, 25)
 
-    def test_get_paging_disabled(self):
-        self.app.config['DOMAIN'][self.known_resource]['pagination'] = False
-        response, status = self.get(self.known_resource, '?page=2')
+    def test_get_custom_page(self):
+        self.app.config['QUERY_PAGE'] = 'custom'
+
+        page = 2
+        response, status = self.get(self.known_resource, '?custom=%d' % page)
         self.assert200(status)
-        resource = response['_items']
-        self.assertFalse(len(resource) ==
-                         self.app.config['PAGINATION_DEFAULT'])
-        self.assertTrue(self.app.config['META'] not in response)
+
         links = response['_links']
-        self.assertTrue('next' not in links)
-        self.assertTrue('prev' not in links)
+        self.assertNextLink(links, 3)
+        self.assertPrevLink(links, 1)
+        self.assertLastLink(links, 5)
+        self.assertPagination(response, 2, 101, 25)
+
+    def test_get_pagination_no_documents(self):
+        """ test that pagination meta is present even when no records are being
+        returned. #415.
+        """
+        response, status = self.get(self.known_resource,
+                                    '?where={"ref": "not_really"}')
+        self.assert200(status)
+        self.assertPagination(response, 1, 0, 25)
 
     def test_get_paging_disabled_no_args(self):
         self.app.config['DOMAIN'][self.known_resource]['pagination'] = False
@@ -102,8 +120,7 @@ class TestGet(TestBase):
 
     def test_get_where_mongo_syntax(self):
         where = '{"ref": "%s"}' % self.item_name
-        response, status = self.get(self.known_resource,
-                                    '?where=%s' % where)
+        response, status = self.get(self.known_resource, '?where=%s' % where)
         self.assert200(status)
 
         resource = response['_items']
@@ -119,6 +136,15 @@ class TestGet(TestBase):
         resource = response['_items']
         self.assertEqual(len(resource), 1)
 
+    def test_get_custom_where(self):
+        self.app.config['QUERY_WHERE'] = 'whereas'
+        where = '{"ref": "%s"}' % self.item_name
+        response, status = self.get(self.known_resource, '?whereas=%s' % where)
+        self.assert200(status)
+
+        resource = response['_items']
+        self.assertEqual(len(resource), 1)
+
     def test_get_mongo_query_blacklist(self):
         where = '{"$where": "this.ref == ''%s''"}' % self.item_name
         _, status = self.get(self.known_resource, '?where=%s' % where)
@@ -159,6 +185,51 @@ class TestGet(TestBase):
         resource = response['_items']
         self.assertEqual(len(resource), 1)
 
+    def test_get_query_in_links(self):
+        """ Make sure that query strings appear in all HATEOAS links (#464).
+        """
+        # find a role with enough results
+        for role in ('agent', 'client', 'vendor'):
+            where = 'role == %s' % role
+            response, _ = self.get(self.known_resource, '?where=%s' % where)
+            if response['_meta']['max_results'] \
+                    >= self.app.config['PAGINATION_LIMIT'] + 1:
+                break
+        links = response['_links']
+        total = response['_meta']['total']
+        max_results = response['_meta']['max_results']
+        last_page = total / max_results + (1 if total % max_results else 0)
+        self.assertTrue('?where=%s' % where in links['self']['href'])
+        self.assertTrue('?where=%s' % where in links['next']['href'])
+        self.assertTrue('?where=%s' % where in links['last']['href'])
+        self.assertNextLink(links, 2)
+        self.assertLastLink(links, last_page)
+
+        page = 2
+        response, _ = self.get(self.known_resource,
+                               '?where=%s&page=%d' % (where, page))
+        links = response['_links']
+        self.assertTrue('?where=%s' % where in links['prev']['href'])
+        self.assertPrevLink(links, 1)
+
+    def test_get_projection_consistent_etag(self):
+        """ Test that #369 is fixed and projection queries return consistent
+            etags (as they are now stored along with the document).
+        """
+        etag_field = self.app.config['ETAG']
+        data = {"inv_number": self.random_string(10)}
+
+        # post a new item so etag storage kicks in
+        r, status = self.post(self.empty_resource_url, data=data)
+        etag = r[etag_field]
+
+        # hit the resource endpoint with a projection query
+        projection = '{"prog": 1}'
+        r, status = self.get(self.empty_resource,
+                             '?projection=%s' % projection)
+        # compare original etag with retrieved one
+        self.assertEqual(etag, r['_items'][0][etag_field])
+
     def test_get_projection(self):
         projection = '{"prog": 1}'
         response, status = self.get(self.known_resource, '?projection=%s' %
@@ -196,6 +267,41 @@ class TestGet(TestBase):
             self.assertTrue(r[self.app.config['LAST_UPDATED']] != self.epoch)
             self.assertTrue(r[self.app.config['DATE_CREATED']] != self.epoch)
 
+    def test_get_custom_projection(self):
+        self.app.config['QUERY_PROJECTION'] = 'view'
+        projection = '{"prog": 1}'
+        response, status = self.get(self.known_resource, '?view=%s' %
+                                    projection)
+        self.assert200(status)
+
+        resource = response['_items']
+
+        for r in resource:
+            self.assertFalse('location' in r)
+            self.assertFalse('role' in r)
+            self.assertTrue('prog' in r)
+
+    def test_get_projection_subdocument(self):
+        projection = '{"location.address": 1}'
+        response, status = self.get(self.known_resource, '?projection=%s' %
+                                    projection)
+        self.assert200(status)
+
+        resource = response['_items']
+
+        for r in resource:
+            self.assertTrue('location' in r)
+            self.assertTrue('address' in r['location'])
+            self.assertFalse('city' in r['location'])
+            self.assertFalse('role' in r)
+            self.assertFalse('prog' in r)
+            self.assertTrue(self.app.config['ID_FIELD'] in r)
+            self.assertTrue(self.app.config['ETAG'] in r)
+            self.assertTrue(self.app.config['LAST_UPDATED'] in r)
+            self.assertTrue(self.app.config['DATE_CREATED'] in r)
+            self.assertTrue(r[self.app.config['LAST_UPDATED']] != self.epoch)
+            self.assertTrue(r[self.app.config['DATE_CREATED']] != self.epoch)
+
     def test_get_projection_noschema(self):
         self.app.config['DOMAIN'][self.known_resource]['schema'] = {}
         response, status = self.get(self.known_resource)
@@ -218,6 +324,17 @@ class TestGet(TestBase):
         resource = response['_items']
         self.assertEqual(len(resource), self.app.config['PAGINATION_DEFAULT'])
 
+    def test_get_sort_comma_delimited_syntax(self):
+        sort = '-prog'
+        response, status = self.get(self.known_resource, '?sort=%s' % sort)
+        self.assert200(status)
+
+        resource = response['_items']
+        self.assertEqual(len(resource), self.app.config['PAGINATION_DEFAULT'])
+        topvalue = 100
+        for i in range(len(resource)):
+            self.assertEqual(resource[i]['prog'], topvalue - i)
+
     def test_get_sort_mongo_syntax(self):
         sort = '[("prog",-1)]'
         response, status = self.get(self.known_resource,
@@ -230,16 +347,29 @@ class TestGet(TestBase):
         for i in range(len(resource)):
             self.assertEqual(resource[i]['prog'], topvalue - i)
 
+    def test_get_custom_sort(self):
+        self.app.config['QUERY_SORT'] = 'orderby'
+        sort = '[("prog",-1)]'
+        response, status = self.get(self.known_resource, '?orderby=%s' % sort)
+        self.assert200(status)
+
+        resource = response['_items']
+        self.assertEqual(len(resource), self.app.config['PAGINATION_DEFAULT'])
+        topvalue = 100
+        for i in range(len(resource)):
+            self.assertEqual(resource[i]['prog'], topvalue - i)
+
     def test_get_sort_disabled(self):
         self.app.config['DOMAIN'][self.known_resource]['sorting'] = False
         sort = '[("prog",-1)]'
-        response, status = self.get(self.known_resource,
-                                    '?sort=%s' % sort)
+        response, status = self.get(self.known_resource, '?sort=%s' % sort)
         self.assert200(status)
         resource = response['_items']
         self.assertEqual(len(resource), self.app.config['PAGINATION_DEFAULT'])
-        for i in range(len(resource)):
-            self.assertEqual(resource[i]['prog'], i)
+
+        # this might actually fail on very rare occurences as mongodb
+        # 'natural' order is not granted to return documents in insertion order
+        self.assertEqual(resource[0]['prog'], 0)
 
     def test_get_default_sort(self):
         s = self.app.config['DOMAIN'][self.known_resource]['datasource']
@@ -256,9 +386,6 @@ class TestGet(TestBase):
         response, _ = self.get(self.known_resource)
         self.assertEqual(response['_items'][0]['prog'], 0)
 
-    def test_get_if_modified_since(self):
-        self.assertIfModifiedSince(self.known_resource_url)
-
     def test_cache_control(self):
         self.assertCacheControl(self.known_resource_url)
 
@@ -353,6 +480,65 @@ class TestGet(TestBase):
             self.assertTrue('_created_on' in document)
             self.assertTrue('_the_etag' in document)
 
+    def test_get_embedded_media(self):
+        """ test that embeedded images are properly rendered and #305 is fixed.
+        """
+
+        # add a 'digital_assets' endpoint to the API
+        self.app.register_resource(
+            'digital_assets',
+            {'schema': {'file': {'type': 'media'}}}
+        )
+
+        # add an 'images' endpoint to the API. this will expose the embedded
+        # digital assets
+        images = {
+            'image_file': {
+                'type': 'objectid',
+                'data_relation': {
+                    'resource': 'digital_assets',
+                    'field': '_id',
+                    'embeddable': True
+                }
+            }
+        }
+        self.app.register_resource('images', {'schema': images})
+
+        # post an asset
+        asset = b'a_file'
+        data = {'file': (BytesIO(asset), 'test.txt')}
+        response, status = self.parse_response(
+            self.test_client.post("digital_assets",
+                                  data=data,
+                                  headers=[('Content-Type',
+                                            'multipart/form-data')]))
+        self.assert201(status)
+
+        # post a document to the 'images' endpoint. the document is referencing
+        # the newly posted digital asset.
+        data = {'image_file': ObjectId(response['_id'])}
+        response, status = self.parse_response(
+            self.test_client.post("images", data=data))
+        self.assert201(status)
+
+        # retrieve the document from the same endpoint, requesting for the
+        # digital asset to be embedded within the retrieved document
+        image_id = response['_id']
+        response, status = self.parse_response(
+            self.test_client.get(
+                '%s/%s%s' % ('images', image_id,
+                             '?embedded={"image_file": 1}')))
+        self.assert200(status)
+
+        # test that the embedded document contains the same data as orignially
+        # posted on the digital_asset endpoint.
+        returned = response['image_file']['file']
+        # encodedstring will raise a DeprecationWarning under Python3.3, but
+        # the alternative encodebytes is not available in Python 2.
+        encoded = base64.encodestring(asset).decode('utf-8')
+        self.assertEqual(returned, encoded)
+        self.assertEqual(base64.decodestring(returned.encode()), asset)
+
     def test_get_embedded(self):
         # We need to assign a `person` to our test invoice
         _db = self.connection[MONGO_DBNAME]
@@ -422,7 +608,22 @@ class TestGet(TestBase):
         content = json.loads(r.get_data())
         self.assertTrue('location' in content['person'])
 
-    def test_get_default_embedding(self):
+        # Add new embeddable field to schema
+        invoices['schema']['missing-field'] = {
+            'type': 'objectid',
+            'data_relation': {'resource': 'contacts', 'embeddable': True}
+        }
+
+        # Test that it ignores embeddable field that is missing from document
+        embedded = '{"missing-field": 1}'
+        r = self.test_client.get('%s/%s' % (invoices['url'],
+                                            '?embedded=%s' % embedded))
+        self.assert200(r.status_code)
+        content = json.loads(r.get_data())
+        self.assertFalse('missing-field' in content['_items'][0])
+
+    def test_get_custom_embedded(self):
+        self.app.config['QUERY_EMBEDDED'] = 'included'
         # We need to assign a `person` to our test invoice
         _db = self.connection[MONGO_DBNAME]
 
@@ -432,46 +633,87 @@ class TestGet(TestBase):
                             {'$set': {'person': fake_contact_id}})
 
         invoices = self.domain['invoices']
-
-        # Turn default field embedding on
-        invoices['embedded_fields'] = ['person']
+        invoices['schema']['person']['data_relation']['embeddable'] = True
 
         # Test that doesn't come embedded if asking for a field that
         # isn't embedded (global setting is False by default)
-        r = self.test_client.get(invoices['url'])
+        embedded = '{"person": 1}'
+        invoices['embedding'] = True
+        r = self.test_client.get('%s/%s' % (invoices['url'],
+                                            '?included=%s' % embedded))
         self.assert200(r.status_code)
         content = json.loads(r.get_data())
-        self.assertTrue(content['_items'][0]['person'], self.item_id)
+        self.assertTrue('location' in content['_items'][0]['person'])
+
+    def test_get_reference_embedded_in_subdocuments(self):
+        _db = self.connection[MONGO_DBNAME]
+
+        contacts = self.random_contacts(2)
+        contact_ids = _db.contacts.insert(contacts)
+        company = {'departments': [{'title': 'development',
+                                   'members': contact_ids}]}
+        company_id = _db.companies.insert(company)
+
+        companies = self.domain['companies']
+        contact_ids = list(map(str, contact_ids))
+
+        # Test that doesn't come embedded if asking for a field that
+        # isn't embedded ('embeddable' is False by default)
+        embedded = '{"departments.members": 1}'
+        r = self.test_client.get('%s/%s' % (companies['url'],
+                                            '?embedded=%s' % embedded))
+        self.assert200(r.status_code)
+        content = json.loads(r.get_data())
+        self.assertEqual(content['_items'][0]['departments'][0]['members'],
+                         contact_ids)
 
         # Set field to be embedded
-        invoices['schema']['person']['data_relation']['embeddable'] = True
+        department_def = companies['schema']['departments']['schema']
+        member_def = department_def['schema']['members']['schema']
+        member_def['data_relation']['embeddable'] = True
 
         # Test that global setting applies even if field is set to embedded
-        invoices['embedding'] = False
-        r = self.test_client.get(invoices['url'])
+        companies['embedding'] = False
+        r = self.test_client.get('%s/%s' % (companies['url'],
+                                            '?embedded=%s' % embedded))
         self.assert200(r.status_code)
         content = json.loads(r.get_data())
-        self.assertTrue(content['_items'][0]['person'], self.item_id)
+        self.assertEqual(content['_items'][0]['departments'][0]['members'],
+                         contact_ids)
 
         # Test that it works
-        invoices['embedding'] = True
-        r = self.test_client.get(invoices['url'])
+        companies['embedding'] = True
+        r = self.test_client.get('%s/%s' % (companies['url'],
+                                            '?embedded=%s' % embedded))
         self.assert200(r.status_code)
         content = json.loads(r.get_data())
-        self.assertTrue('location' in content['_items'][0]['person'])
+        self.assertTrue('location' in
+                        content['_items'][0]['departments'][0]['members'][0])
 
         # Test that it ignores a bogus field
-        invoices['embedded_fields'] = ['person', 'not-really']
-        r = self.test_client.get(invoices['url'])
+        embedded = '{"departments.members": 1, "not-a-real-field": 1}'
+        r = self.test_client.get('%s/%s' % (companies['url'],
+                                            '?embedded=%s' % embedded))
         self.assert200(r.status_code)
         content = json.loads(r.get_data())
-        self.assertTrue('location' in content['_items'][0]['person'])
+        self.assertTrue('location' in
+                        content['_items'][0]['departments'][0]['members'][0])
 
         # Test that it works with item endpoint too
-        r = self.test_client.get('%s/%s' % (invoices['url'], self.invoice_id))
+        embedded = '{"departments.members": 1}'
+        r = self.test_client.get('%s/%s/%s' % (companies['url'], company_id,
+                                               '?embedded=%s' % embedded))
         self.assert200(r.status_code)
         content = json.loads(r.get_data())
-        self.assertTrue('location' in content['person'])
+        self.assertTrue('location' in content['departments'][0]['members'][0])
+
+        # Test default fields to be embedded
+        companies['embedded_fields'] = {"departments.members": 1}
+        r = self.test_client.get('%s/' % companies['url'])
+        self.assert200(r.status_code)
+        content = json.loads(r.get_data())
+        self.assertTrue('location' in
+                        content['_items'][0]['departments'][0]['members'][0])
 
     def test_get_nested_resource(self):
         response, status = self.get('users/overseas')
@@ -569,6 +811,34 @@ class TestGet(TestBase):
         r = self.test_client.get(request, headers=[('Origin', 'test.com')])
         self.assert404(r.status_code)
 
+    def test_get_invalid_where_syntax(self):
+        """ test that 'where' syntax with unknown '$' operator returns 400. """
+        response, status = self.get(self.known_resource,
+                                    '?where={"field": {"$foo": "bar"}}')
+        self.assert400(status)
+
+    def test_get_invalid_sort_syntax(self):
+        """ test that invalid sort syntax returns a 400 """
+        response, status = self.get(self.known_resource, '?sort=[("prog":1)]')
+        self.assert400(status)
+
+    def test_get_allowed_filters_operators(self):
+        """ test that supported operators are not considered invalid filters
+            (#388). Also, test that nested filters are validated.
+        """
+        where = '?where={"$and": [{"field1": "value1"}, {"field2": "value2"}]}'
+        settings = self.app.config['DOMAIN'][self.known_resource]
+
+        # valid
+        settings['allowed_filters'] = ['field1', 'field2']
+        response, status = self.get(self.known_resource, where)
+        self.assert200(status)
+
+        # invalid
+        settings['allowed_filters'] = ['field2']
+        response, status = self.get(self.known_resource, where)
+        self.assert400(status)
+
     def assertGet(self, response, status, resource=None):
         self.assert200(status)
 
@@ -810,7 +1080,7 @@ class TestGetItem(TestBase):
 
         # IMS needs to see as recent as possible since the test db has just
         # been built
-        header = [("If-Modified-Since", date_to_str(datetime.now()))]
+        header = [("If-Modified-Since", date_to_rfc1123(datetime.now()))]
 
         r = self.test_client.get(self.item_id_url, headers=header)
         self.assert304(r.status_code)
@@ -919,6 +1189,24 @@ class TestEvents(TestBase):
         r, s = self.parse_response(self.get_resource())
         self.assertEqual(len(r[self.app.config['ITEMS']]), 1)
 
+    def test_on_pre_GET_resource_dynamic_filter_12_chr_nonunicode_string(self):
+        # Test for bug in _mongotize(). See
+        # https://github.com/nicolaiarocci/eve/issues/508
+        def filter_this(request, lookup):
+            request.args = ImmutableMultiDict(
+                {"where": '{"name":"Alice Brooks"}'}
+            )
+        self.app.register_resource(
+            'names',
+            {'schema': {'name': {'type': 'string'}}}
+        )
+        # We want to test with a non-unicode string for 'where', so we need to
+        # do it with a pre_GET callback
+        self.app.on_pre_GET_names += filter_this
+        self.post('names', data={"name": "Alice Brooks"})
+        r, s = self.get('names')
+        self.assertEqual(len(r[self.app.config['ITEMS']]), 1)
+
     def test_on_pre_GET_resource_for_resource(self):
         self.app.on_pre_GET_contacts += self.devent
         self.get_resource()
