@@ -3,6 +3,7 @@
 from bson import ObjectId
 import copy
 from eve.tests import TestBase
+from eve.tests.utils import DummyEvent
 from eve import STATUS, STATUS_OK, ETAG
 from eve.tests.test_settings import MONGO_DBNAME
 
@@ -32,6 +33,8 @@ class TestVersioningBase(TestBase):
         del(self.domain['contacts']['schema']['title']['default'])
         del(self.domain['contacts']['schema']['dependency_field1']['default'])
         del(self.domain['contacts']['schema']['read_only_field']['default'])
+        del(self.domain['contacts']['schema']['dict_with_read_only']
+                       ['schema']['read_only_in_dict']['default'])
         if partial is True:
             contact_schema = self.domain['contacts']['schema']
             contact_schema[self.unversioned_field]['versioned'] = False
@@ -164,14 +167,31 @@ class TestNormalVersioning(TestVersioningBase):
             shadow_document[self.document_id_field])
         self.assertTrue(self.app.config['ID_FIELD'] in shadow_document)
         self.assertTrue(self.app.config['LAST_UPDATED'] in shadow_document)
+        self.assertTrue(self.app.config['ETAG'] in shadow_document)
 
         # verify that no unexpected fields exist
-        num_meta_fields = 4  # see previous block
+        num_meta_fields = 5  # see previous block
         if partial is True:
             self.assertEqual(len(shadow_document.keys()), num_meta_fields + 1)
         else:
             self.assertEqual(len(shadow_document.keys()), num_meta_fields + 2)
 
+    def assertHateoasLinks(self, links, version_param):
+        """ Makes sure links for `self`, `collection`, and `parent` point to
+        the right place.
+        """
+        self_url = links['self']['href']
+        coll_url = links['collection']['href']
+        prnt_url = links['parent']['href']
+        self.assertTrue('?version=%s' % (str(version_param)) in self_url)
+        if version_param in ('all', 'diffs'):
+            self.assertEqual(self_url.split('?')[0], coll_url)
+            self.assertEqual(coll_url.rsplit('/', 1)[0], prnt_url)
+        else:
+            self.assertEqual('%s?version=all' % self_url.split('?')[0],
+                             coll_url)
+            self.assertEqual(coll_url.split('?')[0], prnt_url)
+
     def do_test_get(self):
         query = '?where={"%s":"%s"}' % \
             (self.app.config['ID_FIELD'], self.item_id)
@@ -203,6 +223,8 @@ class TestNormalVersioning(TestVersioningBase):
         self.assert200(status)
         self.assertDocumentVersionFields(response, 1, 2)
         self.assertEqualFields(version_1, response, self.fields)
+        links = response['_links']
+        self.assertHateoasLinks(links, 1)
 
         # check the get of the second version
         response, status = self.get(self.known_resource, item=self.item_id,
@@ -210,6 +232,8 @@ class TestNormalVersioning(TestVersioningBase):
         self.assert200(status)
         self.assertDocumentVersionFields(response, 2)
         self.assertEqualFields(self.item_change, response, self.fields)
+        links = response['_links']
+        self.assertHateoasLinks(links, 2)
 
         # check the get without version specified and make sure it is version 2
         response, status = self.get(self.known_resource, item=self.item_id)
@@ -362,6 +386,100 @@ class TestCompleteVersioning(TestNormalVersioning):
         self.assertEqual(len(items[1].keys()), len(meta_fields))
         self.assertEqual(items[1][self.app.config['ETAG']], etag2)
 
+        # check the `self` links for both versions
+        self_href = items[0]['_links']['self']['href']
+        self.assertEqual(int(self_href.split('?version=')[1]),
+                         items[0][self.version_field])
+        self_href = items[1]['_links']['self']['href']
+        self.assertEqual(int(self_href.split('?version=')[1]),
+                         items[1][self.version_field])
+
+    def test_getitem_version_pagination(self):
+        """ Verify that `?version=all` and `?version=diffs` display pagination
+        links when results exceed `PAGINATION_DEFAULT`.
+        """
+        # create many versions
+        response, status = self.put(
+            self.item_id_url, data=self.item_change,
+            headers=[('If-Match', self.item_etag)])
+        for n in range(100):
+            response, status = self.put(
+                self.item_id_url, data=self.item_change,
+                headers=[('If-Match', response[self.app.config['ETAG']])])
+
+        # get 2nd page of results
+        page = 2
+        response, status = self.get(self.known_resource, item=self.item_id,
+                                    query='?version=all&page=%d' % page)
+        links = response['_links']
+        self.assertNextLink(links, 3)
+        self.assertPrevLink(links, 1)
+        self.assertLastLink(links, 5)
+        self.assertPagination(response, 2, 102, 25)
+        self.assertHateoasLinks(links, 'all')
+
+    def test_on_fetched_item(self):
+        """ Verify that on_fetched_item events are fired for versioned
+        requests.
+        """
+        devent = DummyEvent(lambda: True)
+        self.app.on_fetched_item += devent
+        response, status = self.get(self.known_resource, item=self.item_id,
+                                    query='?version=1')
+        self.assertEqual(self.known_resource, devent.called[0])
+        self.assertEqual(
+            self.item_id,
+            str(devent.called[1][self.app.config['ID_FIELD']]))
+        self.assertEqual(2, len(devent.called))
+
+        # check for ?version=all requests
+        devent = DummyEvent(lambda: True)
+        self.app.on_fetched_item += devent
+        response, status = self.get(self.known_resource, item=self.item_id,
+                                    query='?version=all')
+        self.assertEqual(self.known_resource, devent.called[0])
+        self.assertEqual(
+            self.item_id,
+            str(devent.called[1][self.app.config['ID_FIELD']]))
+        self.assertEqual(2, len(devent.called))
+
+        # check for ?version=diffs requests
+        devent = DummyEvent(lambda: True)
+        self.app.on_fetched_item += devent
+        response, status = self.get(self.known_resource, item=self.item_id,
+                                    query='?version=diffs')
+        self.assertEqual(None, devent.called)
+
+    def test_on_fetched_item_contacts(self):
+        """ Verify that on_fetched_item_contacts events are fired for versioned
+        requests.
+        """
+        devent = DummyEvent(lambda: True)
+        self.app.on_fetched_item_contacts += devent
+        response, status = self.get(self.known_resource, item=self.item_id,
+                                    query='?version=1')
+        self.assertEqual(
+            self.item_id,
+            str(devent.called[0][self.app.config['ID_FIELD']]))
+        self.assertEqual(1, len(devent.called))
+
+        # check for ?version=all requests
+        devent = DummyEvent(lambda: True)
+        self.app.on_fetched_item_contacts += devent
+        response, status = self.get(self.known_resource, item=self.item_id,
+                                    query='?version=all')
+        self.assertEqual(
+            self.item_id,
+            str(devent.called[0][self.app.config['ID_FIELD']]))
+        self.assertEqual(1, len(devent.called))
+
+        # check for ?version=diffs requests
+        devent = DummyEvent(lambda: True)
+        self.app.on_fetched_item_contacts += devent
+        response, status = self.get(self.known_resource, item=self.item_id,
+                                    query='?version=diffs')
+        self.assertEqual(None, devent.called)
+
         # TODO: also test with HATEOS off
 
     def test_getitem_version_diffs(self):
@@ -491,14 +609,14 @@ class TestCompleteVersioning(TestNormalVersioning):
         self.item_change[self.version_field] = '1'
         r, status = self.post(
             self.known_resource_url, data=self.item_change)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(r, {self.version_field: 'unknown field'})
 
         # set _latest_version
         self.item_change[self.latest_version_field] = '1'
         r, status = self.post(
             self.known_resource_url, data=self.item_change)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(
             r, {self.latest_version_field: 'unknown field'})
 
@@ -506,7 +624,7 @@ class TestCompleteVersioning(TestNormalVersioning):
         self.item_change[self.document_id_field] = '1'
         r, status = self.post(
             self.known_resource_url, data=self.item_change)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(
             r, {self.document_id_field: 'unknown field'})
 
@@ -516,7 +634,7 @@ class TestCompleteVersioning(TestNormalVersioning):
         """
         data = {"person": self.unknown_item_id}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         expected = ("value '%s' must exist in resource '%s', field '%s'" %
                     (self.unknown_item_id, 'contacts',
                      self.app.config['ID_FIELD']))
@@ -539,7 +657,7 @@ class TestCompleteVersioning(TestNormalVersioning):
 
         # delete resource and verify no errors
         response, status = self.delete(self.known_resource_url)
-        self.assert200(status)
+        self.assert204(status)
 
         # verify that there are 0 documents in both collections
         self.assertTrue(self.countDocuments() == 0)
@@ -556,7 +674,7 @@ class TestCompleteVersioning(TestNormalVersioning):
         # delete resource and verify no errors
         response, status = self.delete(
             self.item_id_url, headers=[('If-Match', self.item_etag)])
-        self.assert200(status)
+        self.assert204(status)
 
         # verify that neither primary or shadow documents exist
         self.assertTrue(self.countDocuments(self.item_id) == 0)
@@ -591,25 +709,25 @@ class TestDataRelationVersionNotVersioned(TestNormalVersioning):
         # must be a dict
         data = {"person": self.item_id}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(r, {'person': 'must be of dict type'})
 
         # must have _id
         data = {"person": {value_field: self.item_id}}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(r, {'person': validation_error_format})
 
         # must have _version
         data = {"person": {version_field: 1}}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(r, {'person': validation_error_format})
 
         # bad id format
         data = {"person": {value_field: 'bad', version_field: 1}}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(
             r, {'person': {
                 value_field: "value 'bad' cannot be converted to a ObjectId"}})
@@ -618,7 +736,7 @@ class TestDataRelationVersionNotVersioned(TestNormalVersioning):
         data = {"person": {
             value_field: self.unknown_item_id, version_field: 1}}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(
             r, {'person': "value '%s' must exist in "
                 "resource '%s', field '%s' at version '%s'." %
@@ -627,7 +745,7 @@ class TestDataRelationVersionNotVersioned(TestNormalVersioning):
         # version doesn't exist
         data = {"person": {value_field: self.item_id, version_field: 2}}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(
             r, {'person': "value '%s' must exist in "
                 "resource '%s', field '%s' at version '%s'." %
@@ -695,7 +813,7 @@ class TestDataRelationVersionVersioned(TestNormalVersioning):
         # try saving a field from the first version against version 2
         data = {"person": {'ref': self.item['ref'], self.version_field: 2}}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(
             r, {'person': "value '%s' must exist in "
                 "resource '%s', field '%s' at version '%s'." %
@@ -847,7 +965,7 @@ class TestLateVersioning(TestVersioningBase):
 
         # delete resource and verify no errors
         response, status = self.delete(self.known_resource_url)
-        self.assert200(status)
+        self.assert204(status)
 
         # verify that there are 0 documents in both collections
         self.assertTrue(self.countDocuments() == 0)
@@ -864,7 +982,7 @@ class TestLateVersioning(TestVersioningBase):
         # delete resource and verify no errors
         response, status = self.delete(
             self.item_id_url, headers=[('If-Match', self.item_etag)])
-        self.assert200(status)
+        self.assert204(status)
 
         # verify that neither primary or shadow documents exist
         self.assertTrue(self.countDocuments(self.item_id) == 0)
